<?php
/**
 * @version 2.0 2012-03-07
 * @package Joomla
 * @subpackage Intellectual Property
 * @copyright (C) 2012 the Thinkery
 * @license GNU/GPL see LICENSE.php
 */

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.application.component.controlleradmin');
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.archive');
jimport('joomla.log.log');

ini_set('auto_detect_line_endings',TRUE);

class IpropertyControllerImport_csv extends JControllerAdmin
{
    private $database;
    private $log;
    private $debug;
    private $dumptables;
    private $suppress;
    private $img_path;
    private $cat_column;
    private $amen_column;
    private $id_column;
    private $co_id_column;
    private $agent_id_column     = array();
    private $amenities_array     = array();
    private $state_array         = array();
    private $category_array      = array();
    private $country_array       = array();
    private $tables              = array('iproperty', 'iproperty_companies', 'iproperty_agents', 'iproperty_images');
    private $stype_array;
    private $settings;

    function __construct()
    {
        parent::__construct();

        $app  = JFactory::getApplication();
        $this->database     = JFactory::getDbo();
        $this->settings     = ipropertyAdmin::config();
        $this->stype_array  = ipropertyHTML::get_stypes();       

        if(JRequest::getInt('debug')){
            $this->debug = true;
            JLog::addLogger( array('text_file' => 'iproperty_csv.log.php'));
            if($this->debug) JLog::add('Constructing CSV Import');
        }

        // get image path
        if($this->img_path = JRequest::getString('img_path')){
            if(!$this->img_path = JPath::clean(JPATH_SITE.DS.$this->img_path)){
                $msg = JText::_('IMG_PATH ERROR').' IP Error0: ' . $this->img_path;
                $type = 'notice';
                if($this->debug) JLog::add('Img_path not found or not in site root:' . $this->img_path);
                $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
            }
        }

        // create db connection
        //if(!$this->database = JFactory::getDBO() && ($this->debug)) JLog::add('FAILED TO GET DB OBJECT!');

        // set dumptables var
        $this->dumptables = (JRequest::getVar('empty')) ? true : false;

        // prepare connection for UTF8
        $this->database->setQuery("SET CHARACTER SET 'utf8'");
        if(!$this->database->Query()){
            $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error1 - row ' . $inc . ': ' . $this->database->getErrorMsg();
            $type = 'notice';
            if($this->debug) JLog::add($this->database->getErrorMsg());
            $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
        }

        $this->database->setQuery("SET NAMES 'utf8'");
        if(!$this->database->Query()){
            $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error2 - row ' . $inc . ': ' . $this->database->getErrorMsg();
            $type = 'notice';
            if($this->debug) JLog::add($this->database->getErrorMsg());
            $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
        }

        // create amen array
        $query = "SELECT * FROM #__iproperty_amenities WHERE 1";
        $this->database->setQuery($query);
        if(!$amenities = $this->database->loadObjectList()){
            $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error3 - row ' . $inc . ': ' . $this->database->getErrorMsg();
            $type = 'notice';
            if($this->debug) JLog::add($this->database->getErrorMsg());
            $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
        } else {
            foreach($amenities as $am){
                $this->amenities_array[strtolower($am->title)] = $am->id;
            }
        }

        // create cat array
        $query = "SELECT id, title FROM #__iproperty_categories WHERE 1";
        $this->database->setQuery($query);
        if(!$cats = $this->database->loadObjectList()){
            $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error4 - row ' . $inc . ': ' . $this->database->getErrorMsg();
            $type = 'notice';
            if($this->debug) JLog::add($this->database->getErrorMsg());
            $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
        } else {
            foreach($cats as $am){
                $this->category_array[strtolower($am->title)] = $am->id;
            }
        }

        // create state array
        $query = "SELECT id, mc_name FROM #__iproperty_states WHERE 1";
        $this->database->setQuery($query);
        if(!$states = $this->database->loadObjectList()){
            $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error5 - row ' . $inc . ': ' . $this->database->getErrorMsg();
            $type = 'notice';
            if($this->debug) JLog::add($this->database->getErrorMsg());
            $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
        } else {
            foreach($states as $am){
                $this->state_array[$am->mc_name] = $am->id;
            }
        }

        // create country array
        $query = "SELECT id, mc_name FROM #__iproperty_countries WHERE 1";
        $this->database->setQuery($query);
        if(!$countries = $this->database->loadObjectList()){
            $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error5 - row ' . $inc . ': ' . $this->database->getErrorMsg();
            $type = 'notice';
            if($this->debug) JLog::add($this->database->getErrorMsg());
            $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
        } else {
            foreach($countries as $am){
                $this->country_array[$am->mc_name] = $am->id;
            }
        }

    }

    function cancel()
    {
        // Check for request forgeries
        JRequest::checkToken() or die( 'Invalid Token' );
        $this->setRedirect( 'index.php?option=com_iproperty' );
    }

    function importCSV()
    {
        $app  = JFactory::getApplication();
        $option     = JRequest::getCmd('option');

        $_rows        = 0;
        $msg          = null;

        // Check for request forgeries
        JRequest::checkToken() or die( 'Invalid Token' );

        if(!JRequest::getVar('csv_file')){
            $msg = JText::_( 'COM_IPROPERTY_NO_FILE_SELECTED' );
            $type = 'notice';
            if($this->debug) JLog::add('No file selected');
            $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
            return false;
        } else {
            if($this->debug) JLog::add('Beginning import_csv function');
        }

        $csv_file      = JPATH_SITE.DS.'media'.DS.'com_iproperty'.DS.JRequest::getVar('csv_file');

        // open the csv file and attempt to read it in
        if (!$file_handle = fopen($csv_file, "r")) {
            $msg = sprintf(JText::_( 'COM_IPROPERTY_COULD_NOT_READ_CSV' ), $csv_file);
            $type = 'notice';
            if($this->debug) JLog::add('COULD NOT READ CSV');
            $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
            return false;
        }

        // now empty the tables of all existing IP data
        if ($this->dumptables){
            if($this->debug) JLog::add('Executing emptying query');
            $emptying_query = '';

            $emptying_query .= 'TRUNCATE TABLE #__iproperty;';
            $emptying_query .= 'TRUNCATE TABLE #__iproperty_agentmid;';
            $emptying_query .= 'TRUNCATE TABLE #__iproperty_agents;';
            $emptying_query .= 'TRUNCATE TABLE #__iproperty_companies;';
            $emptying_query .= 'TRUNCATE TABLE #__iproperty_images;';
            $emptying_query .= 'TRUNCATE TABLE #__iproperty_propmid;';

            $this->database->setQuery ($emptying_query);
            if(!$this->database->QueryBatch()) {
                $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error6 - row ' . $inc . ': ' . $this->database->getErrorMsg();
                $type = 'notice';
                if($this->debug) JLog::add('IP ERROR 6');
                $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
                return false;
            }
        }

        ################################################################
        # THIS IS WHERE THE IMPORT HAPPENS                             #
        ################################################################
        $inc = 0;

        while (($row = fgetcsv($file_handle, '', ',', '"')) !== FALSE) {
            if($this->debug) JLog::add('Beginning import');
            if ($inc == 0) { // this is the first row so we use it to map the values
                if($this->debug) JLog::add('Creating arrays');
                foreach ($row as $r => $v) {
                    $columnname = explode("|", $v);
                    //var_dump( $columnname  );
                    if(isset($columnname[2])){
                        $str = "$" . $columnname[0] . "[" . $columnname[2] . "][" . $columnname[1] . "]=" . $r . ";";
                    } else {
                        $str = "$" . $columnname[0] . "[" . $columnname[1] . "]=" . $r . ";";
                    }
                    //echo $str;
                    eval($str);
                }

                if(is_array($iproperty)){
                    // set the cat_column to the column # of the category, since we'll need to unset this
                    $this->cat_column = $iproperty['category'];
                    unset($iproperty['category']);

                    // same for amenities
                    $this->amen_column = $iproperty['amenities'];
                    unset($iproperty['amenities']);

                    // same for id
                    $this->id_column = $iproperty['id'];
                    $iproperty['ip_source'] = $iproperty['id'];
                    $iproperty['state'] = 10000;
                    unset($iproperty['id']);

                }

                if(is_array($iproperty_agents)){
                    foreach($iproperty_agents as $ag => $agent){
                        // set the id column to the column # of the category, since we'll need to unset this
                        $this->agent_id_column[$ag] = $iproperty_agents[$ag]['id'];
                        unset($iproperty_agents[$ag]['id']);
                        $iproperty_agents[$ag]['company'] = 100001;
                    }
                }

                if(is_array($iproperty_companies)){
                    // set the id column to the column # of the category, since we'll need to unset this
                    $this->co_id_column = $iproperty_companies['id'];
                    unset($iproperty_companies['id']);
                }

            } else { // we're past the header row, so now we do the data handling

                if($this->debug) JLog::add('Beginning insert row ' . $inc);

                ########################################################
                # COMPANY INSERT                                       #
                ########################################################
                $query_string = '';
                // rename the id field
                $iproperty_companies['ip_source'] = $this->co_id_column;

                // get country
                $row[$iproperty_companies['country']] = $this->country_array[$row[$iproperty_companies['country']]];

                // get locstate
                $row[$iproperty_companies['locstate']] = $this->state_array[$row[$iproperty_companies['locstate']]];

                // do the image transform
                if($row[$iproperty_companies['icon']]) $this->doAgentThumb_gd($row[$iproperty_companies['icon']], 'companies');
                $row[$iproperty_companies['icon']] = JFile::makeSafe($row[$iproperty_companies['icon']]);

                foreach( $iproperty_companies as $column => $value ){
                    $query_string .= $column . "=" . $this->database->Quote(trim($row[$value])) . ",";
                }
                
                // set state to 3
                $query_string .= ' state = 3,';

                $query_string = rtrim( $query_string, ',' );

                $query = "INSERT INTO #__iproperty_companies SET " . $query_string . " ON DUPLICATE KEY UPDATE " . $query_string . ", id = LAST_INSERT_ID(id)";

                $this->database->setQuery($query);
                if(!$this->database->Query()){
                    $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error7 - row ' . $inc . ': ' . $this->database->getErrorMsg();
                    $type = 'notice';
                    if($this->debug) JLog::add($this->database->getErrorMsg());
                    $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
                    return false;
                }

                // get last insert id and set as agent company id
                $new_company = $this->database->insertid();

                // end company insert

                ########################################################
                # AGENT INSERT                                         #
                ########################################################
                $new_agents = array();
                foreach( $iproperty_agents as $m => $agent ){
                    if ($row[$agent['fname']] != ''){

                        $query_string = '';
                        // rename the id field
                        $agent['ip_source'] = $this->agent_id_column[$m];
                        $row[$agent['company']] = $new_company;

                        // get country
                        $row[$agent['country']] = $this->country_array[$row[$agent['country']]];

                        // get state
                        $row[$agent['locstate']] = $this->state_array[$row[$agent['locstate']]];

                        // do agent image transform
                        if($row[$agent['icon']]) $this->doAgentThumb_gd($row[$agent['icon']], 'agents');
                        $row[$agent['icon']] = JFile::makeSafe($row[$agent['icon']]);

                        foreach ( $agent as $column => $value){
                            $query_string .= $column . "=" . $this->database->Quote(trim($row[$value])) . ",";
                        }
                        
                        // set state to 3
                        $query_string .= ' state = 3,';
                        
                        $query_string = rtrim( $query_string, ',' );

                        $query = "INSERT INTO #__iproperty_agents SET " . $query_string . " ON DUPLICATE KEY UPDATE " . $query_string . ", id = LAST_INSERT_ID(id)";

                        $this->database->setQuery($query);
                        if(!$this->database->Query()){
                            $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error8 - row ' . $inc . ': ' . $this->database->getErrorMsg();
                            $type = 'notice';
                            if($this->debug) JLog::add($this->database->getErrorMsg());
                            $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
                            return false;
                        }

                        // get last insert id and add to agent array for use in mid_table
                        $new_agents[] = $this->database->insertid();
                    }
                } // end agent insert

                ########################################################
                # PROPERTY INSERT                                      #
                ########################################################
                $query_string = '';
                // reset the company id
                $row[$iproperty['listing_office']] = $new_company;
                // rename the id column
                $row[$iproperty['ip_source']] = $row[0];

                // set state = 3
                $row[$iproperty['state']] = 3;

                // pull the category var
                $cat = $this->category_array[strtolower($row[$this->cat_column])] ? $this->category_array[strtolower($row[$this->cat_column])] : 0;

                // pull the amenity var
                $amen = $row[$this->amen_column];

                // if there's no lat/lon values, do the geocoder
                if (!$row[$iproperty['latitude']] || !$row[$iproperty['longitude']]) {
                    $prop_address   = '';
                    $prop_address  .= $row[$iproperty['street_num']] ? trim($row[$iproperty['street_num']]) . ' ' : '';
                    $prop_address  .= $row[$iproperty['street']] ? trim($row[$iproperty['street']]) : '';
                    $prop_address  .= $row[$iproperty['apt']] ? ', ' . trim($row[$iproperty['apt']]) : '';
                    $prop_address  .= $row[$iproperty['locstate']] ? ', ' . trim($row[$iproperty['locstate']]) : '';
                    $prop_address  .= $row[$iproperty['province']] ? ', ' . trim($row[$iproperty['province']]) : '';
                    $prop_address  .= $row[$iproperty['country']] ? ', ' . trim($row[$iproperty['country']]) : '';

                    if (($location = $this->doGeocode($prop_address)) !== FALSE){
                        $row[$iproperty['latitude']]   = $location[0];
                        $row[$iproperty['longitude']]  = $location[1];
                    }
                }

                // get country
                $row[$iproperty['country']] = $this->country_array[$row[$iproperty['country']]];

                // get locstate
                $row[$iproperty['locstate']] = $this->state_array[$row[$iproperty['locstate']]];

                // get stype
                $row[$iproperty['stype']] = array_search($row[$iproperty['stype']], $this->stype_array);

                // build the query
                foreach( $iproperty as $column => $value ){
                    $query_string .= $column . "=" . $this->database->Quote(trim($row[$value])) . ",";
                }

                $query_string = rtrim( $query_string, ',' );

                $query = "INSERT INTO #__iproperty SET " . $query_string . " ON DUPLICATE KEY UPDATE " . $query_string . ",id=LAST_INSERT_ID(id)";
                $this->database->setQuery($query);
                if(!$this->database->Query()){
                    $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error9 - row ' . $inc . ': ' . $this->database->getErrorMsg();
                    $type = 'notice';
                    if($this->debug) JLog::add($this->database->getErrorMsg());
                    $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
                    return false;
                }

                // get last insert id
                $new_property = $this->database->insertid();

                // end property insert

                ########################################################
                # IMAGE INSERT                                         #
                ########################################################
                $k = 1;
                foreach( $iproperty_images as $image ){
                    if($row[$image['fname']] != ''){

                        $image['ordering'] = 99999;
                        $image['path']     = 99998;
                        $image['propid']   = 99997;
                        $image['type']     = 99996;

                        // add the prop_id field
                        $row[$image['propid']]   = $new_property;
                        $row[$image['ordering']] = $k;

                        // parse out the fname value into the parts we need
                        $path                         = $row[$image['fname']];
                        $path_parts                   = pathinfo($path);
                        $row[$image['fname']]         = JFile::makeSafe($path_parts['filename']);
                        // account for naming issues with image files
                        switch(strtolower($path_parts['extension'])){
                            case 'jpg':
                                $row[$image['type']] = '.jpg';
                                break;
                            case 'jpeg':
                                $row[$image['type']] = '.jpg';
                                break;
                            case 'jpe':
                                $row[$image['type']] = '.jpg';
                                break;
                            case 'png':
                                $row[$image['type']] = '.png';
                                break;
                            default:
                                $row[$image['type']] = '.jpg';
                                break;
                        }
                        // If it's not remote we need to resize
                        if (!$row[$image['remote']]){
                            $this->doThumb_gd($path);
                        } else {
                            $row[$image['path']] = dirname($path) . '/';
                        }

                        $query_string = '';
                        foreach ( $image as $column => $value){
                            $query_string .= $column . "=" . $this->database->Quote(trim($row[$value])) . ",";
                        };

                        $query_string = rtrim( $query_string, ',' );
                        $query = "INSERT INTO #__iproperty_images SET " . $query_string;

                        $this->database->setQuery($query);
                        if(!$this->database->Query()){
                            $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error10 - row ' . $inc . ': ' . $this->database->getErrorMsg();
                            $type = 'notice';
                            if($this->debug) JLog::add($this->database->getErrorMsg());
                            $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
                            return false;
                        }

                        $k++;
                    }
                } // end image insert

                ########################################################
                # AGENT MID INSERT                                     #
                ########################################################
                foreach ($new_agents as $agent){
                    $query = "INSERT INTO #__iproperty_agentmid (prop_id, agent_id) VALUES (" . $new_property . "," . $agent . ")";
                    $this->database->setQuery($query);
                    if(!$this->database->Query()){
                        $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error11 - row ' . $inc . ': ' . $this->database->stderr();
                        $type = 'notice';
                        if($this->debug) JLog::add($this->database->getErrorMsg());
                        $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
                        return false;
                    }
                }

                ########################################################
                # PROP MID INSERT                                      #
                ########################################################
                $query = "INSERT INTO #__iproperty_propmid (prop_id, cat_id) VALUES (" . $new_property . "," . $cat . ")";
                $this->database->setQuery($query);
                if(!$this->database->Query()){
                    $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error12 - row ' . $inc . ': ' . $this->database->getErrorMsg();
                    $type = 'notice';
                    if($this->debug) JLog::add($this->database->getErrorMsg());
                    $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
                    return false;
                }

                ########################################################
                # AMENITIES INSERT                                     #
                ########################################################
                if(strlen($amen) > 0){
                    $temp_amen = explode(',', $amen);
                    foreach ($temp_amen as $amen){
                        $tamen = $this->amenities_array[strtolower($amen)];
                        if($tamen){
                            $query = "INSERT INTO #__iproperty_propmid (prop_id, amen_id) VALUES (" . $new_property . "," . $this->amenities_array[strtolower($amen)] . ")";
                            $this->database->setQuery($query);
                            if(!$this->database->Query()){
                                $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error13 - row ' . $inc . ': ' . $this->database->getErrorMsg();
                                $type = 'notice';
                                if($this->debug) JLog::add($this->database->getErrorMsg());
                                $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
                                return false;
                            }
                        }
                    }
                }
            } // end of row import
            $inc++; // increment row count
        } // end of fgetcsv

        ########################################################
        # CLEANUP QUERY                                        #
        ########################################################

        // here is where we'll set the ip_source values back to NULL
        // set state = 1 for all properties where state = 3
        // etc.
        foreach($this->tables as $table){
            $query = "UPDATE #__" . $table . " SET state = 1 WHERE state = 3";
            $this->database->setQuery($query);
            if(!$this->database->Query()){
                $msg = JText::_( 'COM_IPROPERTY_QUERIES_EXECUTION_FAILED' ).' IP Error14 - row ' . $inc . ': ' . $this->database->getErrorMsg();
                $type = 'notice';
                if($this->debug) JLog::add($this->database->getErrorMsg());
                $app->redirect('index.php?option=com_iproperty&view=import_csv', $msg, $type);
                return false;
            }
        }

        // finished imports-- redirect and close
        fclose($file_handle);
        $type = 'message';
        $msg  = 'Import finished-- ' . ($inc - 1) . ' rows imported from csv.';
        if($this->debug) JLog::add($msg);

        $app->redirect('index.php?option=com_iproperty', $msg, $type);

        parent::display();
    }

    ########################################################
    # HELPER FUNCTIONS                                     #
    ########################################################

    private function doGeocode($address)
    {
        if($this->debug) JLog::add("Doing geocode for " . $address);
        $mapaddress = urlencode( $address );

        // Desired address
        $url = "http://maps.google.com/maps/geo?q=$mapaddress&output=xml";

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); //Set curl to return the data instead of printing it to the browser.
        curl_setopt($ch, CURLOPT_URL, $url);

        $page = curl_exec($ch);
        curl_close($ch);

        //sleep(1);

        // Parse the returned XML file
        try{
            $xml = @new SimpleXMLElement($page);
        } catch (Exception $e) {
            if($this->debug) JLog::add("SimpleXML error in geocode function: " . $e);
            return false;
        }
        // Parse the coordinate string
        list($longitude, $latitude, $altitude) = explode(",", $xml->Response->Placemark->Point->coordinates);
        $g_address = addslashes($xml->Response->Placemark->address);
        $status = $xml->Response->Status->code;
        $google_coordinates = $xml->Response->Placemark->Point->coordinates;

        unset($xml);
        unset($page);

        if ($status == 200) {
            if($this->debug) JLog::add("Geocode found-- " . $g_address . " latitude: " . $latitude . " longitude: " . $longitude);
            $location = array($latitude, $longitude);
            return $location;
        } elseif ( $status == 602 || $status == 603 ) { // no address or private address
            if($this->debug) JLog::add("No geocode found for property at " . $address);
            return false;
        } else {
            if ($status == 620 && !$this->suppress) {
                if($this->debug) JLog::add("OUT OF GEOCODES! FURTHER ERRORS SUPPRESSED.");
                $this->suppress = true;
            } elseif ($status != 620) {
                if($this->debug) JLog::add("A problem with the geocoder occured. Error: " . $status);
            }
            return false;
        }
    }

    // GD THUMB FUNCTION
    private function doThumb_gd($img_name)
    {
        // pass in the filename
        $thumb_w = $this->settings->thumbwidth ? $this->settings->thumbwidth : 150;
        $thumb_h = $this->settings->thumbheight ? $this->settings->thumbheight : 95;
        $fullsize_w = $this->settings->imgwidth ? $this->settings->imgwidth : 710;
        $fullsize_h = $this->settings->imgheight ? $this->settings->imgheight : 450;
        $filename = $this->img_path . DS . $img_name;

        if(!JFile::exists($filename)){
            if($this->debug) JLog::add("ERROR in GD transform: file does not exist.");
            return false;
        }

        try{
            $system=explode(".",$img_name);
                $shortname = JFile::makeSafe($system[0]);
                try{
                    if (preg_match("/jpg|jpeg/i",$system[1])) {
                        if (!$src_img=imagecreatefromjpeg($filename))  throw new Exception('Could not create jpg');
                    }
                } catch (Exception $e) {
                    if($this->debug) JLog::add("ERROR 1 in GD transform: " . $e);
                    return;
                }
                try{
                    if (preg_match("/png/i",$system[1])) {
                        if(!$src_img=imagecreatefrompng($filename))  throw new Exception('Could not create png.');
                    }
                } catch (Exception $e) {
                    if($this->debug) JLog::add("ERROR 2 in GD transform: " . $e);
                    return;
                }
                if($this->debug) JLog::add("Doing GD transform for " . $filename);
        } catch (Exception $e) {
            if($this->debug) JLog::add("ERROR 3 in GD transform: " . $e);
            return;
        }

        $old_x=imageSX($src_img);
        $old_y=imageSY($src_img);

        if($this->settings->imgproportion){ // we need to keep originally proportions so get ratio
            $ratio         = $old_y / $old_x;

            $thumb_h       = $ratio * $thumb_w;
            $fullsize_h    = $ratio * $fullsize_w;

            $th_w = $thumb_w;
            $th_h = $thumb_h;
            $fs_w = $fullsize_w;
            $fs_h = $fullsize_h;
        }

        if(!$this->debug) unlink( $filename ); // delete the temp images only if not in debug mode
        // Output the image
        // create fullsize

            try{
                $dst_img=ImageCreateTrueColor($fs_w,$fs_h);
                $newfile = JPATH_SITE . $this->settings->imgpath . $shortname . ".jpg";
                imagecopyresampled($dst_img,$src_img,0,0,0,0,$fs_w,$fs_h,$old_x,$old_y);
                imagejpeg($dst_img,$newfile);
                if($this->debug) JLog::add("Created " . $newfile);
            }
            catch(Exception $e) {
                if($this->debug) JLog::add("ERROR 2 in GD transform for " . $filename . " - " . $e);
            }
            //imagedestroy($dst_img);
         // create thumb
            try{
                $dst_img=ImageCreateTrueColor($th_w,$th_h);
                $newfile = JPATH_SITE . $this->settings->imgpath . $shortname . "_thumb.jpg";
                imagecopyresampled($dst_img,$src_img,0,0,0,0,$th_w,$th_h,$old_x,$old_y);
                imagejpeg($dst_img,$newfile);
                if($this->debug) JLog::add("Created " . $newfile);
            }
            catch(Exception $e) {
                if($this->debug) JLog::add("ERROR 3 in GD transform for " . $filename . " - " . $e);
            }
            imagedestroy($dst_img);
            imagedestroy($src_img);
    }

    // GD THUMB FUNCTION
    private function doAgentThumb_gd($img_name, $folder)
    {
        // pass in the filename
        if ($folder == 'agents'){
            $thumb_w = $this->settings->agentphotowidth ? $this->settings->agentphotowidth : 90;
        } elseif ($folder == 'companies') {
            $thumb_w = $this->settings->company_photo_width ? $this->settings->company_photo_width : 90;
        } else {
            return false;
        }

        $filename = $this->img_path . DS . $img_name;

        if (!JFile::exists($filename)){
            if($this->debug) JLog::add("ERROR in GD " . $folder . " transform. File does not exist: " . $filename . " - " . $img_name);
            return false;
        }

        try{
            $system=explode(".",$img_name);
                $shortname = JFile::makeSafe($system[0]);
                try{
                    if (!$src_img=imagecreatefromjpeg($filename))  throw new Exception('Could not create jpg');
                } catch (Exception $e) {
                    if($this->debug) JLog::add("ERROR 1 in GD " . $folder . " transform: " . $e);
                    return;
                }
                if($this->debug) JLog::add("Doing GD " . $folder . " transform for " . $filename);
        } catch (Exception $e) {
            if($this->debug) JLog::add("ERROR 2 in GD " . $folder . " transform: " . $e);
            return;
        }

        $old_x=imageSX($src_img);
        $old_y=imageSY($src_img);

        $ratio         = $old_y / $old_x;
        $thumb_h       = $ratio * $thumb_w;
        $th_w = $thumb_w;
        $th_h = $thumb_h;

        if(!$this->debug) unlink( $filename ); // delete the temp images only if not in debug mode
        // Output the image
        // create thumb
            try{
                $dst_img=ImageCreateTrueColor($th_w,$th_h);
                $newfile = JPATH_SITE . DS . 'media' . DS . 'com_iproperty' . DS . $folder . DS . $shortname . ".jpg";
                imagecopyresampled($dst_img,$src_img,0,0,0,0,$th_w,$th_h,$old_x,$old_y);
                imagejpeg($dst_img,$newfile);
                if($this->debug) JLog::add("Created " . $newfile);
            }
            catch(Exception $e) {
                if($this->debug) JLog::add("ERROR 3 in GD " . $folder . " transform for " . $filename . " - " . $e);
            }
            imagedestroy($dst_img);
            imagedestroy($src_img);
    }
}
?>