var PropertyWidget = new Class({

	Implements: [Events, Chain, Options],

	Binds: ['googleCallback', 'requestComplete', 'createMarker'],

	options: {
		ipbaseurl: '',
		sliderLength: 300,
		Itemid: 99999,
		showPreview: 1,
		noLimit: 0,
		startPage: 1,
		currencySeparator: ',',
		currencySymbol: '$',
		currencyPosition: 1,
		marker: '/components/com_iproperty/assets/images/map/icon56.png',
        token: '',
        thumbnail: '',
        slideColor: '#64992C',
        mapWidth: 710,
        mapHeight: 350,
        mapPreviewIcon: '/components/com_iproperty/assets/images/map/map_preview.png',
        propPreviewIcon: '/components/com_iproperty/assets/images/map/prop_preview.png',
        saveSearch: false,
        radiusSearch: false,
		text: {
			tprop: 'Results',
			price: 'Price',
			nolimit: 'No Limit',
			pid: 'Property ID',
			street: '<div>Street<span class="street_preview">(Click address to view listing)</span></div>',
			beds: 'Beds',
			baths: 'Baths',
			sqft: 'Ft<sup>2</sup>',
			preview: 'Preview',
			more: 'more',
			inputText: 'ID or Keyword',
			noRecords: 'Sorry, no records were found. Please try again.',
			previous: '&#706 Previous ',
			next: 'Next &#707',
			of: 'of',
            searchopt: 'More Criteria',
            savesearch: 'Save search',
            clearsearch: 'Clear search'
		},
		map: {
			zoom: 12,
			maxZoom: 12,
			streetViewControl: false,
			mapTypeId: google.maps.MapTypeId.ROADMAP,
			lat: '47.6725282',
			lng: '-116.7679661'
		},
		search: {
			city: '',
            locstate: '',
            province: '',
            region: '',
            country: '',
			stype: '',
			limit: 10,
			limitstart: 0,
			search: '',
			option:'com_iproperty',
			view: 'advsearch',
			task:'ajax.ajaxSearch',
			ptype: '',
            waterfront: '',
            reo: '',
			hoa: '',
			format: 'raw'
		},
		inputs: {
        
		},
		sliders: {

		},
		templates:{
			slider: '<div class="property_slider">' +
						'<div class="slider_labels">' +
							'<span class="slider_label_min">{nolimit}</span>{title}<span class="slider_label_max">{nolimit}</span>' +
						'</div>' +
						'<div class="slider_element pressed">' +
							'<div class="slider_knob slider_knob_start"></div>' +
							'<div class="slider_knob slider_knob_end"></div>' +
						'</div>' +
					'</div>',
			infoWindow: '<div class="bubble">' +
							'{thumb}' +
							'<h4><a href="{proplink}">{street_address}, {city}</a></h4>' +
							'<div class="bubble_info"><strong>{pid}: </strong>{mls_id} | <strong>{price}: </strong>{formattedprice}</div>' +
							'<div class="bubble_desc">{short_description}<div class="bubble_cats">{cat_icons}</div><a href="{proplink}">({more})</a></div>' +
						'</div>',
			pager: '<li class="page_range">{pagecount}</li><li class="page_buttons">{previous}{next}</li>',
			pageButton: '<div class="page_button gradient-button {class}" style="display: {display};" />{value}</div>'
		}
	},

	initialize: function(element, options){ 
        // check if search cookie is set
        this.ipsearchCookie = new Hash.Cookie('ipAdvSearch'+options.Itemid);

        if ( this.ipsearchCookie.getLength() ){        
            this.setCookieOptions(this.ipsearchCookie, options);
        }
        
		this.setOptions(options);
		this.element = $(element);    
		this.markers = {};
		this.sliders = [];
		this.inputs = [];
		this.query = {};
        this.centerMarker = null;
        this.centerCircle = null;
        this.searchRadius = null;
        this.centerRad = null;
        this.centerLatLng = null;
		this.page = this.options.search.limitstart / this.options.search.limit || 1;
		this.scroll = new Fx.Scroll(window, { duration: 400, transition: 'quad:out', offset: { y: -2 } });

		this.mapElement = new Element('div', {
			id: 'property_map',
			styles: {
				height: this.options.mapHeight,
                width: this.options.mapWidth
			}
		});

        // build the save search element in the toolbar
        this.ipSave = new Element('span', { id: 'ipSaveSearch' }); 
        
		this.slidersElement = new Element('div', { id: 'property_sliders' });
        
        // build div for "clear search" Button
        this.ipClearButton = new Element('div', { id: 'ipClearButton' }).inject(this.slidersElement);
        
		this.attributesPanel = new Element('div', {
			id: 'property_attributes',
			html: '<div class="property_attributes_wrap"><div id="property_attributes_inputs" class="property_attributes_inputs"></div></div><div id="property_attributes_button" class="property_attributes_button gradient-button">'+this.options.text.searchopt+'</div>',
			events: {
				'click:relay(div.property_attributes_button)': function(){
					var inputs = this.getPrevious().getFirst();
					if(!this.hasClass('pressed')) {
						this.addClass('pressed');
						inputs.reveal();
					}
					else{
						this.removeClass('pressed');
						inputs.dissolve();
					}
				}
			}
		});
		this.attributesPanel.getFirst().getFirst().set('reveal', { duration: 250, transition: 'circ:out' });
		this.propertyList = new Element('div', {id: 'property_list'});
		this.element.adopt([this.mapElement, this.slidersElement, this.attributesPanel, this.propertyList]);

		$each(this.options.inputs, function(v, k){
			this.addInput(k, v);
		}, this)

		$each(this.options.sliders, function(v, k){
			this.addSlider(k, v);
		}, this)
        
        this.createMap();

		this.request = new Request.JSON({
			method: 'get',
			url: this.options.ipbaseurl + 'index.php?'+this.options.token+'=1',
			onComplete: this.requestComplete
		});
        
        if (Object.keys(this.options.sliders).length == 0){
            $('property_attributes_button').addClass('pressed');
            $('property_attributes_inputs').reveal();
        }

		this.search();

	},

	createMap: function(){
		this.options.map.center = new google.maps.LatLng(this.options.map.lat, this.options.map.lng);
		this.mapInstance = new google.maps.Map(this.mapElement, this.options.map);
		this.infoWindow = new google.maps.InfoWindow({ maxWidth: 450 });
		this.mapSpinner = new Element('div',{id:'loading_div'}).inject(this.mapElement);
		this.mapCounter = new Element('span', { id: 'property_counter' }).inject(this.mapElement);
        this.ipSave.inject(document.getElementById('ip_toolbar'));
        if (this.options.radiusSearch == true) this.addRadiusListener();
	},

	createMarker: function(house) {
		if(house.lat_pos && house.long_pos && (house.show_map == 1)){
			var latlong = new google.maps.LatLng(house.lat_pos.toFloat(), house.long_pos.toFloat()),
			    marker = new google.maps.Marker({
					position: latlong,
					map: this.mapInstance,
					icon: this.options.ipbaseurl+this.options.marker
				}),
				html = this.getMarkerHtml(house);

			this.markers[house.id] = [marker, html];

			google.maps.event.addListener(marker, 'click', function() {
				this.openInfoWindow(house.id);
			}.bind(this));

			this.bounds.extend(latlong);

			return marker;
		}
		else return false;
	},

	getMarkerHtml: function(house) {
		return this.options.templates.infoWindow.substitute(
			$merge(house, {
				street_address: house.street_address.clean(),
				city: house.city.clean(),
				short_description: house.short_description.slice(0,185).trim() + '...',
				thumb: ('<div class="bubble_image"><a href="{proplink}">{thumb}</a>{banner}</div>').substitute(house),
				pid: this.options.text.pid,
				price: this.options.text.price,
				more: this.options.text.more,
                nolimit: this.options.text.nolimit,
                cat_icons: house.caticons.slice(1)
			})
		);
	},

	addInput: function(title, options){     
		var self = this,
			inputWrap = this.attributesPanel.getFirst().getFirst(),
			change = function(){
			if(self.request){
				self.page = 1;
				self.search();
			}
			},
			input = new Element(options.tag, $merge({ 'title': options.title }, options, {
				'events': $H(options.events || {}).map(function(fn){ return function(e){ fn.call(self, e, this) }; })
			})).inject(
				(options.group) ? ($('property_fieldset_' + options.group) || new Element('fieldset', { id: 'property_fieldset_' + options.group }).inject(inputWrap)) : inputWrap
			);

		if(options.type == 'checkbox' && Browser.Engine.trident);
		else input.addEvent('change', change);

		switch(options.type || options.tag){
			case 'select':
				[options.title, $H(options.value).getKeys()].flatten().each(function(option){
					iptempinput = new Element('option', {
						'text': option,
						'value': (options.value[option]) ? options.value[option] : ''
					});
                    if (options.selected == options.value[option]) {
                        iptempinput.set('selected', 'selected');
                    }
                    iptempinput.inject(input)
				});
			break;
			case 'checkbox':
                title = options.title ? options.title : title;
				iptempinput = new Element('label', {
					text: title,
					value: options.value,
					events: {
						'mouseup': function(){
							if(Browser.Engine.trident){
								input.checked = !input.checked;
								change();
								input.checked = !input.checked;
					}
						}
					}
				});
                if (options.selected){
                    input.checked = true;
                }
                iptempinput.wraps(input)
                
            break;
		}

		this.inputs.push(input);

		return this;
	},

	addSlider: function(title, options){
		var elements = Elements.from(this.options.templates.slider.substitute({title: options.title}))[0].inject(this.slidersElement),
			slider = new Slider.Extra(elements.getElement('.slider_element'),{rangeBackground: this.options.slideColor}),
			minLabel = elements.getElement('.slider_label_min'),
			maxLabel = elements.getElement('.slider_label_max');

		slider.addRange($merge({
			start: {
				knob: elements.getElement('.slider_knob_start'),
				onChange: function(step){
					var opt = this.options;
					minLabel.set('text', (opt.noLimit && opt.range.contains(this.previousChange)) ? window.langOptions.nolimit : (opt.labelUnit || '') + step);
				}
			},
			end: {
				knob: elements.getElement('.slider_knob_end'),
				onChange: function(step){
					var opt = this.options;
					maxLabel.set('text', (opt.noLimit && opt.range.contains(this.previousChange)) ? window.langOptions.nolimit : (opt.labelUnit || '') + step);
				}
			},
			onComplete: function(){
				if(this.request){
					this.page = 1;
					this.search();
				}
			}.bind(this)
		}, options));

		this.sliders.push(slider);

		return this;
	},

	createTable: function(data) {

		var options = this.options.search,
			tableHeaders = [];

		['price', 'pid', 'street', 'beds', 'baths', 'sqft', 'preview'].each(function(e){
			tableHeaders.push({ content: this.options.text[e] });
		}, this);

		this.table = new HtmlTable({
			properties: {
				'id': 'prop_table'
			},
			headers: tableHeaders
		}).enableSort();

		var infoOpener = function(e){
			this.openInfoWindow(e.target.get('resultid'))
		}.bind(this);

		this.table.toElement().addEvents({
			'mouseover:relay(img[preview=mouseover])': infoOpener,
            'mouseover:relay(a[preview=mouseover])': infoOpener,
			'click:relay(a[preview=click])': function(e){
				this.scroll.toElement(this.element).chain(function(){ infoOpener(e) });
			}.bind(this)
		}).inject(this.propertyList);

		this.createPaging();

		return this;
	},

	createPaging: function(){
		var pagingClick = {
				'mousedown:relay(div.page_button)': function(){
					this.addClass('pressed');
				},
				'mouseup:relay(div.page_button)': function(){
					this.removeClass('pressed');
				},
				'click:relay(div.previous_page)': function(e){
					if(this.page > 1){
						this.page--;
						this.search();
					}
				}.bind(this),
				'click:relay(div.next_page)': function(e){
					if(this.checkLimit(this.page + 1)){
						this.page++;
						this.search();
					}
				}.bind(this)
			},
			table = this.table.toElement();

		this.pagers = [
			new Element('ul', {
				'class': 'property_pager',
				'events': pagingClick
			}).inject(table, 'before')
		];

		this.pagers.push(this.pagers[0].clone().addEvents(pagingClick).inject(table, 'after'));

		return this;
	},
	/*
	formatCurrency: function(num) {
		var symbol = this.options.currencySymbol,
			split = (num + '').split('.'),
			wholes = split[0],
			rgx = /(\d+)(\d{3})/;

		while (rgx.test(wholes)) wholes = wholes.replace(rgx, '$1' + this.options.currencySeparator + '$2');

		num = wholes + split.length > 1 ? '.' + split[1] : '';

		return (this.options.currencyPosition == 1) ? num + ' ' + symbol : symbol + num;
	},
	*/
	checkLimit: function(page){
		var limit = this.options.search.limit,
			max = page * limit;

		return (page > 0 && this.totalCount >= max - limit &&  max >= this.totalCount) ? true : false;
	},

	getSliderValues: function(){
		var query = {};
		this.slidersElement.getElements('div.slider_knob').retrieve('slider').each(function(slider){
			var opt = slider.options;
			query[opt.parameter] = (opt.noLimit && opt.range.contains(slider.previousChange)) ? '' : slider.previousChange;
		});
		return query;
	},

	getInputValues: function(){
		var query = {};
		this.attributesPanel.getElements('[parameter]').each(function(input){
			var param = input.get('parameter'),
				type = input.get('type')
				isCustom = !!input.get('custom'),
				value = (isCustom) ? ((query[param]) ? ',' : '') + input.value : (type == 'checkbox') ? 1 : input.value,
				blank = (isCustom) ? '' : 0;
                // unset the value of the select list if it's the same as the title/default
			// The following line is way more hard-coded that it should be; required to accomodate specific API choices for ptype parameter values
			query[param] = (query[param] || '') + ((type == 'checkbox') ? ((input.checked) ? value : blank) : value);
		});
		return query;
	},
    
    saveSearch: function(){
        var ipSearchData = new Hash.Cookie('ipAdvSearch'+this.options.Itemid);
        if(ipSearchData.getLength()){
            // add clear button
            this.addClearButton();
            // create save drawer
            var saveSlide = (document.getElementById('save-panel')) ? new Fx.Slide('save-panel').hide() : null;
            if(saveSlide && (this.options.saveSearch == 1)){           
                this.ipSave.set('html', '<a href="javascript:(0)">'+this.options.text.savesearch+'</a>');
                document.id('ipsavesearchstring').set('value', ipSearchData.toQueryString());
                this.ipSave.addEvent('click', function(e){
                    document.id('save-panel').setStyle('display', 'block');
                    e = new Event(e);
                    (saveSlide.open) ? saveSlide.slideOut() : saveSlide.slideIn();
                    e.stop();
                });
            } else {
                //console.log(document.getElementById('save-panel'));
            }
        }
    },

	search: function(){
		this.mapSpinner.show();
		this.query = $merge(this.options.search, { //get the value of the form elements associated with the search options
				limitstart: this.page * this.options.search.limit - this.options.search.limit
			},
			this.getSliderValues(),
			this.getInputValues()
		);
        
        // add radius search vars
        if (this.centerLatLng && this.centerRad && this.options.radiusSearch){
            this.query.lat = this.centerLatLng.lat();
            this.query.lon = this.centerLatLng.lng();
            this.query.rad = this.centerRad;
        }        

        // remove unused query params to compress cookie size
        var cleanQuery = Object.filter(this.query, function(value, key){
          if (value == 0) return;
          return value;
        });
        
        // delete unnecessary items from cookie to compress further
        delete cleanQuery.limit;
        delete cleanQuery.limitstart;
        delete cleanQuery.task;
        delete cleanQuery.view;
        delete cleanQuery.option;
        delete cleanQuery.format;
        
        // set the current search vars as cookie
        this.ipsearchCookie.empty(); // clean out the cookie        
        var searchCookie = new Hash.Cookie('ipAdvSearch'+this.options.Itemid);
        searchCookie.extend(cleanQuery);
        
		this.request.send({data: this.query});
	},

	requestComplete: function(data){
		this.fireEvent('requestComplete');
		this.results = data;
		this.totalCount = data[0].totalcount;
		this.mapCounter.set('html', this.totalCount || 0);
		this.updateMap(data);
		this.updateTable(data);
		this.updatePaging(data);
		this.mapSpinner.hide();
        this.saveSearch();
	},

	updateMap: function(data){
		$each(this.markers, function(marker, id, markers){
			marker[0].setMap(null);
			delete markers[id];
		});

		this.bounds = new google.maps.LatLngBounds();
		data.each(this.createMarker);
		if(this.totalCount > 0 && !this.centerLatLng) this.mapInstance.fitBounds(this.bounds);

		return this;
	},

	updateTable: function(data){
		if(!this.table) this.createTable(data);

        this.table.push(['']); // add new dummy row to workaround mootools bug
		this.table.empty();

		var options = this.options.search,
			tableRows = [];

		if (this.totalCount > 0) {
			data.each(function(e, i){
				var hasMarker = this.markers[e.id],
					row = [
						'<div class="ip_left price">' + e.formattedprice + '</div>',
						'<div class="ip_centered mls">' + e.mls_id + '</div>',
						'<a resultid="' + e.id + '" href="' + e.proplink + '" ' + ((this.options.showPreview == 1 && hasMarker) ? 'preview="mouseover"' : '') + '>' + e.street_address.clean() + ', ' + e.city.clean() + '</a>',
						'<div class="ip_centered beds">' + e.beds + '</div>',
						'<div class="ip_centered baths">' + e.baths + '</div>',
						'<div class="ip_centered sqft">' + e.sqft + '</div>',
                        '<div class="ip_centered preview">' +
                        //show marker if available
                        ((hasMarker) ? '<img src="' + this.options.ipbaseurl + this.options.mapPreviewIcon+'" alt="' + this.options.text.preview + '" resultid="' + e.id + '" href="#preview_'+ e.id +'" preview="mouseover" style="cursor: pointer;" />&nbsp;' : '') +
                        //show thumbnail if enabled and available
                        ((this.options.thumbnail && !e.thumb.match('nopic.png')) ? '<img src="' + this.options.ipbaseurl + this.options.propPreviewIcon+'" alt="' + this.options.text.preview + '" onmouseover="tooltip.show(\''+addSlashes(e.thumb)+'\');" onmouseout="tooltip.hide();" style="cursor: pointer;" />&nbsp;' : '') +
                        //no marker and no thumbnail - display '--'
                        (((!hasMarker && !this.options.thumbnail) || (!hasMarker && (this.options.thumbnail && e.thumb.match('nopic.png')))) ? '--' : '') +
                        '</div>'
					];
				tableRows.push(row);

			}, this);
		}
		else {
			tableRows.push([{
				content: this.options.text.noRecords,
				properties: {
					'colspan': 7,
					'align': 'center'
				}
			}]);
		}

		tableRows.each(function(row){ this.table.push(row) }, this);

		return this;
	},

	updatePaging: function(data){
		var options = this.options.search,
			prevLimit = (this.page * options.limit - options.limit).limit(0, this.totalCount),
			nextLimit = (this.page * options.limit).limit(0, this.totalCount),
			pagingData = {
				pagecount: this.options.text.tprop + ': ' + prevLimit + ' - ' + nextLimit + ' ' + this.options.text.of + ' ' + this.totalCount,
				previous: this.options.templates.pageButton.substitute({
					'class': 'previous_page',
					'value': this.options.text.previous,
					'display': (!prevLimit) ? 'none' : 'block'
				}),
				next: this.options.templates.pageButton.substitute({
					'class': 'next_page',
					'value': this.options.text.next,
					'display': (nextLimit == this.totalCount) ? 'none' : 'block'
				})
			};

		this.pagers.each(function(pager){
			pager.set('html', this.options.templates.pager.substitute(pagingData));
		}, this);

		return this;
	},

	openInfoWindow: function(id) {
		var marker = this.markers[id];
		this.infoWindow.setContent(marker[1])
		this.infoWindow.open(this.mapInstance, marker[0]);
	},
    
	addClearButton: function(){
        var self = this;
        if(!$('ipClearSearch')){
            var ipClearSearch = new Element('a', {
                id: 'ipClearSearch',
                html: this.options.text.clearsearch,
                href: 'javascript:void(0);',
                styles: {
                    display: 'block'
                },
                events: {
                    click: function(){
                        self.ipsearchCookie.empty();
                        //Cookie.dispose('ipAdvSearch');
                        this.dissolve();
                        window.location.reload(); // works but it's not elegant
                    }
                }
            }).inject(this.ipClearButton);
        }
    },
    
    setCookieOptions: function(ipsearchCookie, options){
        if(ipsearchCookie.has('beds_low')) options.sliders.beds.start.initialStep = ipsearchCookie.get('beds_low');
        if(ipsearchCookie.has('beds_high')) options.sliders.beds.end.initialStep = ipsearchCookie.get('beds_high');
        if(ipsearchCookie.has('baths_low')) options.sliders.baths.start.initialStep = ipsearchCookie.get('baths_low');
        if(ipsearchCookie.has('baths_high')) options.sliders.baths.end.initialStep = ipsearchCookie.get('baths_high');
        if(ipsearchCookie.has('price_low')) options.sliders.price.start.initialStep = ipsearchCookie.get('price_low');
        if(ipsearchCookie.has('price_high')) options.sliders.price.end.initialStep = ipsearchCookie.get('price_high');
        if(ipsearchCookie.has('sqft_low')) options.sliders.sqft.start.initialStep = ipsearchCookie.get('sqft_low');
        if(ipsearchCookie.has('sqft_high')) options.sliders.sqft.end.initialStep = ipsearchCookie.get('sqft_high');
        if(ipsearchCookie.has('province')) options.inputs.province.selected = ipsearchCookie.get('province');
        if(ipsearchCookie.has('region')) options.inputs.region.selected = ipsearchCookie.get('region');
        if(ipsearchCookie.has('city')) options.inputs.city.selected = ipsearchCookie.get('city');
        if(ipsearchCookie.has('county')) options.inputs.county.selected = ipsearchCookie.get('county');
        if(ipsearchCookie.has('locstate')) options.inputs.locstate.selected = ipsearchCookie.get('locstate');
        if(ipsearchCookie.has('country')) options.inputs.country.selected = ipsearchCookie.get('country');
        if(ipsearchCookie.has('reo')) options.inputs.reo.selected = true;
        if(ipsearchCookie.has('hoa')) options.inputs.hoa.selected = true;
        if(ipsearchCookie.has('waterfront')) options.inputs.waterfront.selected = true;
        if(ipsearchCookie.has('stype')) options.inputs.stype.selected = ipsearchCookie.get('stype');
        if(ipsearchCookie.has('ptype')){
            ipsearchCookie.get('ptype').split(',').each(function(e){
                options.inputs['cat'+e].selected = true;
            });
        }    
        if(ipsearchCookie.search) options.search['search'] = ipsearchCookie.search;
        
        return options;
    },
    
    drawCircle: function(){
        var self        = this;
        // only draw the circle if we've got a centerpoint defined
        if(this.centerLatLng){
            if(this.centerCircle) this.centerCircle.setMap(null); // reset circle
            var t_rad       = document.getElementById('centerRadius').value ? document.getElementById('centerRadius').value : 1000; 
            this.centerRad  = Number( t_rad ) / 1000; // get the value in KM to set for main search
            this.centerCircle = new google.maps.Circle({ 
                    map: self.mapInstance, 
                    center: self.centerLatLng, 
                    radius: Number( t_rad ), // meters, not KM
                    fillColor: "#2b86f9", 
                    fillOpacity: 0.2, 
                    strokeColor: "#2b86f9", 
                    strokeOpacity: 1.0, 
                    strokeWeight: 1, 
                    clickable: false 
                });
        }
    },
    
    addRadiusListener: function(){
        var self = this;
        // first remove the change event from the radius dropdown
        document.id('centerRadius').removeEvents('change');
        // attach a new change event to the radius dropdown
        document.id('centerRadius').addEvent('change', function(){
            self.drawCircle();
            self.search();
        });
        // add the map listener for the click event
        google.maps.event.addListener(this.mapInstance, 'click', function(event) {
            self.centerLatLng = event.latLng;
            if(self.centerMarker) self.centerMarker.setMap(null); // reset the marker
            self.centerMarker = new google.maps.Marker({ // draw new marker
                position: self.centerLatLng,
                map: self.mapInstance
            });
            
            self.drawCircle(); // draw the radius circle
            self.search(); // execute search
        });
    }
});

function addSlashes(str) {
    str=str.replace(/\"/g,'\'');
    str=str.replace(/\\/g,'\\\\');
    str=str.replace(/\'/g,'\\\'');
    str=str.replace(/\"/g,'\\"');
    str=str.replace(/\0/g,'\\0');
    return str;
}

var tooltip = function(){
    var id = 'tt';
    var top = 3;
    var left = 3;
    var maxw = 300;
    var speed = 10;
    var timer = 20;
    var endalpha = 95;
    var alpha = 0;
    var tt,t,c,b,h;
    var ie = document.all ? true : false;
    return{
        show:function(v,w){
            if(tt == null){
                tt = document.createElement('div');
                tt.setAttribute('id',id);
                t = document.createElement('div');
                t.setAttribute('id',id + 'top');
                c = document.createElement('div');
                c.setAttribute('id',id + 'cont');
                b = document.createElement('div');
                b.setAttribute('id',id + 'bot');
                tt.appendChild(t);
                tt.appendChild(c);
                tt.appendChild(b);
                document.body.appendChild(tt);
                tt.style.opacity = 0;
                tt.style.filter = 'alpha(opacity=0)';
                document.onmousemove = this.pos;
            }
            tt.style.display = 'block';
            c.innerHTML = v;
            tt.style.width = w ? w + 'px' : 'auto';
            if(!w && ie){
                t.style.display = 'none';
                b.style.display = 'none';
                tt.style.width = tt.offsetWidth;
                t.style.display = 'block';
                b.style.display = 'block';
            }
            if(tt.offsetWidth > maxw){tt.style.width = maxw + 'px'}
            h = parseInt(tt.offsetHeight) + top;
            clearInterval(tt.timer);
            tt.timer = setInterval(function(){tooltip.fade(1)},timer);
        },
        pos:function(e){
            var u = ie ? event.clientY + document.documentElement.scrollTop : e.pageY;
            var l = ie ? event.clientX + document.documentElement.scrollLeft : e.pageX;
            tt.style.top = (u - h) + 'px';
            tt.style.left = (l + left) + 'px';
        },
        fade:function(d){
            var a = alpha;
            if((a != endalpha && d == 1) || (a != 0 && d == -1)){
                var i = speed;
                if(endalpha - a < speed && d == 1){
                    i = endalpha - a;
                }else if(alpha < speed && d == -1){
                    i = a;
                }
                alpha = a + (i * d);
                tt.style.opacity = alpha * .01;
                tt.style.filter = 'alpha(opacity=' + alpha + ')';
            }else{
                clearInterval(tt.timer);
                if(d == -1){tt.style.display = 'none'}
            }
        },
        hide:function(){
            clearInterval(tt.timer);
            tt.timer = setInterval(function(){tooltip.fade(-1)},timer);
        }
    };
}();
